package com.iflytek.cloud;

import android.os.Bundle;


/**
 * <h3>评测回调接口</h3>
 * <p>通过实现此接口，获取当前评测的状态和结果</p>
 * <p>识别监听器，请参考{@link RecognizerListener}</p>
 * <p>合成监听器，请参考{@link SynthesizerListener}</p>
 * <p>语音语义监听器，请参考{@link SpeechUnderstanderListener}</p>
 * <p>文本语义监听器，请参考{@link TextUnderstanderListener}</p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 */
public abstract interface EvaluatorListener {
	
	/**
	 * <h4>音量变化</h4>
	 * <p>
	 *     当开始识别，到停止录音（停止写入音频流）或SDK返回最后一个结果自动结束识别为止，
	 *  SDK检测到音频数据（正在录音或写入音频流）的音量变化时，会多次通过此函数回调，告
	 *  知应用层当前的音量值。应用层可通过此函数传入的值变化，改变自定义UI的画面等。
	 * </p> 
	 * @param volume 当前音量值，范围[0-30]
	 * @param data 录音数据，格式请参考{@link SpeechConstant#SAMPLE_RATE}
	 * 		, {@link SpeechConstant#AUDIO_FORMAT}。此参数返回的数据，即使
	 * {@link SpeechConstant#NOTIFY_RECORD_DATA}为false，也会返回。
	 * 
	 * @see SpeechEvaluator#startEvaluating
	 * @see #onBeginOfSpeech
	 * @see #onEndOfSpeech
	 * @see SpeechEvaluator#stopEvaluating
	 * @see SpeechConstant#NOTIFY_RECORD_DATA
	 */
	void onVolumeChanged(int volume , byte[] data);

	/**
	 * <h4>开始说话</h4>
	 * <p>
	 * 		在录音模式({@link SpeechConstant#AUDIO_SOURCE 音频源}参数设为 > -1时 )下，
	 * 调用{@link SpeechEvaluator#startEvaluating 开始录音函数}后，会自动开启系统的录音
	 * 机，并在录音机开启后，会回调此函数（这中间的过程应该在几毫秒内，可以忽略，除非系
	 * 统响应很慢）。
	 * </p>
	 * <p>
	 * 		应用层可通过此函数回调，告知用户，当前可开始说话。若应用层使用的是声音提示，
	 * 则应该在调用{@link SpeechEvaluator#startEvaluating 开始录音函数}前，提放提示音，
	 * 并在提示音非静音数据播放结束时，调用
	 * {@link SpeechEvaluator#startEvaluating 开始录音函数}，开始录音。若太早调用，则
	 * 可能会把提示音录进要识别的音频中，若太晚，则可能会漏掉部分用户说话的音频。
	 * </p>
	 * @see SpeechEvaluator#startEvaluating
	 * @see #onEndOfSpeech
	 */
	void onBeginOfSpeech();

	/**
	 * <h4>结束说话</h4>
	 * <p>
	 * 		在SDK检测到音频的静音端点时，回调此函数（在录音模式或写音频模式下都会回调，
	 * 应用层主动调用{@link SpeechEvaluator#stopEvaluating}则不会回调此函数，
	 * 在识别出错时，可能不会回调此函数）。在此函数回调后，当前识别会话可能并没有结束，
	 * 识别结果可能还要等待一定的时间才会返回。
	 * </p>
	 * <p>
	 * 		此函数回调后，应用层应立即停止调用{@link SpeechEvaluator#writeAudio}写入音
	 * 频数据，(当音频源设置为音频流时（{@link SpeechConstant#AUDIO_SOURCE}为-1时)
	 * 否则，再通过{@link SpeechEvaluator#writeAudio}写入的音频也会被忽略。
	 * </p>
	 * <p>
	 * 		应用层可以通过此函数回调，告知用户，当次说话已结束，正在等待识别结果（若结果
	 * 未返回）等。
	 * </p>
	 * 
	 * @see #onBeginOfSpeech
	 * @see #onResult
	 * @see #onError
	 * @see SpeechEvaluator#writeAudio
	 * @see SpeechEvaluator#stopEvaluating
	 */
	void onEndOfSpeech();

	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。
	 * </p>
	 * <p>
	 * 		一次会话的结果可能会多次返回（即多次回调此函数），通过参数2，判断是否是最后
	 * 一个结果，true时为最后一个结果，否则不是。当最后一个结果返回时，本次会话结束，录
	 * 音也会停止，在重新调用 {@link SpeechEvaluator#startEvaluating}开启新的会话前，
	 * 停止调用{@link SpeechEvaluator#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * 当出现错误，或应用层调用{@link SpeechEvaluator#cancel}取消当次会话时，在当次
	 * 会话过程可能不会回调此函数。
	 * </p>
	 * <p>
	 * 		会话过程采用边录边上传的分次上传音频数据方式，可能在结束录音前，就有结果返回。
	 * </p>
	 * 
	 * @param result 结果数据
	 * @param islast 是否最后一次结果标记
	 * @see #onError
	 * @see SpeechEvaluator#cancel
	 * @see EvaluatorResult
	 */
	void onResult(EvaluatorResult result, boolean islast);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，录音也会停止。应在再次调用
	 * {@link SpeechEvaluator#startEvaluating}开启新的会话前，
	 * 停止调用{@link SpeechEvaluator#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * </p>
	 * @param error 错误类型
	 * @see SpeechError
	 * @see #onResult
	 * @see SpeechEvaluator#startEvaluating
	 */
	void onError(SpeechError error);

    /**
     * <h4>事件</h4>
     * <p>
     * 		扩展用接口，由具体业务进行约定。例如eventType为0显示网络状态，agr1为网络连接值。
     * </p>
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 消息内容
     * 
     * @see SpeechEvent
     */
	void onEvent(int eventType, int arg1,int arg2, Bundle obj);
	
}
