package com.iflytek.cloud.util;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import android.text.TextUtils;

import com.iflytek.cloud.LexiconListener;
import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * <h3>用户词表类</h3>
 * <p>
 * 		用户个性化词表类，解析、生成json格式数据。用于在听写着上传个性化数据，以提高匹配率，
 * 关于个性化数据上传，参考{@link com.iflytek.cloud.SpeechRecognizer#updateLexicon}。
 * 关于联系人列表获取，参考{@link ContactManager}。
 * </p>
 * <p>
 * 		格式示例如下：
 * <br>{
 * <br>&nbsp;"userword":
 *		<br>&nbsp;&nbsp;[
 *			<br>&nbsp;&nbsp;&nbsp;{ "name" : "default" , "words" : [ "默认词条1", "默认词条2" ] },
 *			<br>&nbsp;&nbsp;&nbsp;{ "name" : "词表名称1", "words": [ "词条1的第一个词", "词条1的第二个词"] },
 *	        <br>&nbsp;&nbsp;&nbsp;{ "name" : "词表名称2", "words": [ "词条2的第一个词", "词条2的第二个词"] }
 *		<br>&nbsp;&nbsp;]
 * <br>}
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * 
 * @see com.iflytek.cloud.SpeechRecognizer#updateLexicon
 * @see ContactManager
 */
public class UserWords {
	
	private static final String USERWORD = "userword";
	
	private static final String DEFAULT_KEY = "default";
	
	private static final String WORDS    = "words";
	
	private static final String NAME     = "name";
	
	// 词表对象
	private Hashtable<String, ArrayList<String>>   mwords = null;
	
	/**
	 * <h4>默认构造函数</h4>
	 */
	public UserWords() {
		mwords = new Hashtable<String, ArrayList<String>>();	
	}

	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		通过传入JSON格式字符串来初始化用户词表
	 * </p>
	 * 
	 * @param json 格式化的用户词表字符串。
	 */
	public UserWords(String json) {
		mwords = new Hashtable<String, ArrayList<String>>();	
		parseFromJson(json);
	}
	
	/**
	 * <h4>是否包含某个键</h4>
	 * <p>
	 * 		是否包含名称为key的词表。
	 * </p>
	 * 
	 * @param key 键名
	 * @return false：否; true：是。
	 */
	public boolean hasKey(String key) {
		return mwords.containsKey(key);
	}

	/**
	 * <h4>添加词</h4>
	 * <p>
	 * 		向默认词组中增加一个value，key值为default。
	 * </p>
	 * 
	 * @param value 需要插入的词条内容
	 * @return true表示插入成功，false表示失败
	 */
	public boolean putWord(String value)
	{
		return putWord(DEFAULT_KEY, value);
	}
	
	/**
	 * <h4>添加词</h4>
	 * <p>
	 * 		向key词组中增加一个value。
	 * </p>
	 * 
	 * @param key 需要插入的词组名称
	 * @param value  需要插入的词条内容
	 */
	public boolean putWord(String key,String value)
	{
		if(TextUtils.isEmpty(key) || TextUtils.isEmpty(value))
			return false;
		ArrayList<String> list = getWords(key);
		if(list != null) // 追加到已存在列表
			appendWord(list,value);
		else			 // 新建列表
		{
			list = new ArrayList<String>();
			appendWord(list,value);
			mwords.put(key, list);
		}
		return true;
	}
	
	/**
	 * <h4>添加多个词</h4>
	 * <p>
	 * 		 向默认词组中增加多个value，key值为default。
	 * </p>
	 *
	 * @param words 需要插入的词条列表
	 * @return true表示插入成功，false表示失败
	 */
	public boolean putWords(ArrayList<String> words)
	{
		return putWords(DEFAULT_KEY, words);
	}
	
	/**
	 * <h4>添加多个词</h4>
	 * <p>
	 * 		 向key词组中增加多个value。
	 * </p>
	 * 
	 * @param key  需要插入的词组名称。
	 * @param words  需要插入的词条内容。
	 * @return true表示插入成功，false表示失败
	 */
	public boolean putWords(String key,ArrayList<String> words)
	{
		if(TextUtils.isEmpty(key) || words == null )
			return false;
		ArrayList<String> list = getWords(key);
		if(list != null) // 追加到已存在列表
			appendWords(list, words);
		else			 // 新建列表
		{
			list = new ArrayList<String>();
			appendWords(list, words);
			mwords.put(key,list);
		}
		return true;
	}
	
	/**
	 * <h4>追加词</h4>
	 * <p>
	 * 		向词组列表中追加一个词条。
	 * </p>
	 * 
	 * @param list 词组列表
	 * @param word 需要追加的词条
	 * @return true表示插入成功，false表示失败
	 */
	private boolean appendWord(ArrayList<String> list,String word)
	{
		if(list != null && !list.contains(word))
		{
			list.add(word);
			return true;
		}
		return false;
	}

	/**
	 * <h4>追加多个词</h4>
	 * <p>
	 * 		向词组列表中追加多个词条。
	 * </p>
	 * 
	 * @param list 词组列表
	 * @param words 需要追加的词条列表
	 * @return true表示插入成功，false表示失败
	 */
	private boolean appendWords(ArrayList<String> list,ArrayList<String>  words)
	{
		if(list == null || words == null)
			return false;
		for(String word : words)
		{
			appendWord(list, word);
		}
		return true;
	}
	
	/**
	 * <h4>获取词条</h4>
	 * <p>
	 * 		获取默认词组下的所有词条。
	 * </p>
	 * 
	 * @return 默认词组的词条列表
	 */
	public ArrayList<String> getWords()
	{
		return getWords(DEFAULT_KEY);
	}
	
	/**
	 * <h4>获取键值</h4>
	 * <p>
	 * 		 获取词表下的所有键值。
	 * </p>
	 *
	 * @return 词表下的所有键值
	 */
	public ArrayList<String> getKeys()
	{
		if(mwords == null || mwords.isEmpty())
			return null;
		ArrayList<String> keys = new ArrayList<String>();
		Iterator<String> iterator= mwords.keySet().iterator();
		while(iterator.hasNext())
			keys.add(iterator.next());
		return keys;
	}
	
	/**
	 * <h4>获取词条</h4>
	 * <p>
	 * 		获取键名key下，对应所有用户词表。
	 * </p>
	 * 
	 * @param key 需要获取的词组名称。
	 * @return 词条列表
	 */
	public ArrayList<String> getWords(String key)
	{
		return mwords.get(key);
	}
	
	/**
	 * <h4>解析用户词表json格式数据</h4>
	 * <p>
	 * 		与{@link #UserWords(String)}相当。
	 * </p>
	 * @param json 用户词表json格式字符串
	 */
	private void parseFromJson(String json)
	{
		try {
			if(TextUtils.isEmpty(json))
			{
				DebugLog.LogD("userword is null...");
				return;
			}
			
			JSONObject object = new JSONObject(new JSONTokener(json));
			JSONArray userwords = object.getJSONArray(USERWORD);
			for (int i = 0; i < userwords.length(); i++) {
				JSONObject userword  = (JSONObject) userwords.get(i);
				JSONArray words = userword.getJSONArray(WORDS);
				ArrayList<String> wordArray = new ArrayList<String>();
				for(int j = 0; j < words.length(); j++){
					String word = words.get(j).toString();
					if(wordArray != null && !wordArray.contains(word))
						wordArray.add(word);
				}
				mwords.put(userword.get(NAME).toString(), wordArray);
			}
		} catch (JSONException e) {
			DebugLog.LogE(e);
		}
		
	}
	
	/**
	 * <h4>转为json字符串</h4>
	 * <p>
	 * 		将用户词表转为字符串，用于
	 * {@link com.iflytek.cloud.SpeechRecognizer#updateLexicon(String, String, LexiconListener)}上传。
	 * </p>
	 * 
	 * @return 用户词表的字符串
	 */
	private String toJson() 
	{
		try{
			if(null == mwords)
			{
				DebugLog.LogD("mwords is null...");
				return null;
			}
			
			JSONObject object = new JSONObject();
			JSONArray userwords = new JSONArray();
			
			Iterator<Entry<String, ArrayList<String>>> itors = mwords.entrySet().iterator();
			while(itors.hasNext())
			{
				Map.Entry<String, ArrayList<String>> entry = itors.next();
				JSONObject userword = new JSONObject();
				JSONArray words = new JSONArray();
				for(String word : entry.getValue())
					words.put(word);
				userword.put(WORDS, words);
				userword.put(NAME, entry.getKey());
				userwords.put(userword);
			}
			
			object.put(USERWORD, userwords);
			return object.toString();
		}catch (JSONException e) {
			DebugLog.LogE(e);
			return null;
		}
	}
	
	/**
	 * <h4>转为String</h4>
	 * <p>
	 * 		同{@link #toJson()}
	 * </p>
	 * 
	 * @see #toJson()
	 */
	@Override
	public String toString() {
		return toJson();
	}
}
